<?php
/* --------------------------------------------------------------
	CleverReachController.inc.php 2016-03-14
	Gambio GmbH
	http://www.gambio.de
	Copyright (c) 2015 Gambio GmbH
	Released under the GNU General Public License (Version 2)
	[http://www.gnu.org/licenses/gpl-2.0.html]
	--------------------------------------------------------------
*/

class CleverReachController extends HttpViewController
{
	/**
	 * module configuration
	 * @var CleverReachConfigurationStorage
	 */
	protected $configuration;

	/**
	 * true if Module Center Module is installed
	 * @var bool
	 */
	protected $enabled;

	/**
	 * Logger
	 * @var CleverReachLogger
	 */
	protected $logger;

	public function __construct(HttpContextReaderInterface $httpContextReader,
	                            HttpResponseProcessorInterface $httpResponseProcessor,
	                            ContentViewInterface $contentView)
	{
		parent::__construct($httpContextReader, $httpResponseProcessor, $contentView);
		$this->configuration = MainFactory::create('CleverReachConfigurationStorage');
		$this->enabled       = (bool)gm_get_conf('MODULE_CENTER_CLEVERREACH_INSTALLED') === true;
		$this->logger        = MainFactory::create('CleverReachLogger');
	}


	public function actionDefault()
	{
		return new RedirectHttpControllerResponse(xtc_href_link('index.php'));
	}

	public function actionCron()
	{
		if($this->enabled === true)
		{
			$output = '';
			if($this->_getQueryParameter('key') === $this->configuration->get('cron_pass'))
			{
				try
				{
					$crSync  = MainFactory::create('CleverReachSync');
					$newsletterGroupID = $this->configuration->get('newsletter_group_id');
					if($newsletter_group_id > 0)
					{
						$crGroup = MainFactory::create('CleverReachGroup', $newsletter_group_id);
						$this->logger->notice(sprintf('CRON syncing %d entries from queue', $crSync->getNumberOfQueueEntries()));
						$crSync->syncUpstreamQueue(null, $crGroup);
						$this->logger->notice('CRON performing complete newsletter sync');
						$crSync->completeNewsletterSync($crGroup);
					}
					$customersGroupID = $this->configuration->get('allcustomers_group_id');
					if($customersGroupID > 0)
					{
						$customersGroup = MainFactory::create('CleverReachGroup', $customersGroupID);
						$this->logger->notice(sprintf("CRON syncing customers to group %s", $customersGroup->name));
						$crSync->sendAllCustomers($customersGroup);
					}
					$buyersGroupID = $this->configuration->get('allorders_group_id');
					if($buyersGroupID > 0)
					{
						$buyersGroup = MainFactory::create('CleverReachGroup', $buyersGroupID);
						$this->logger->notice(sprintf("CRON adding buyers to group %s", $buyersGroup->name));
						$crSync->sendAllBuyers($buyersGroup);
					}
					$output = 'CRON finished';
				}
				catch (Exception $e)
				{
					$this->logger->notice('CRON encountered an error: ' . $e->getMessage());
					$output = 'ERROR, see log file';
				}
			}
			else
			{
				$this->logger->notice(sprintf('CRON called from IP %s with an invalid key', $_SERVER['REMOTE_ADDR']));
				$output = 'INVALID ACCESS (logged)';
			}
			return new HttpControllerResponse($output);
		}
		else
		{
			return new RedirectHttpControllerResponse(xtc_href_link('index.php'));
		}
	}

}


