<?php
/* --------------------------------------------------------------
	CleverReachCheckoutSuccessExtender.inc.php 2016-04-12
	Gambio GmbH
	http://www.gambio.de
	Copyright (c) 2015 Gambio GmbH
	Released under the GNU General Public License (Version 2)
	[http://www.gnu.org/licenses/gpl-2.0.html]
	--------------------------------------------------------------
*/

class CleverReachCheckoutSuccessExtender extends CleverReachCheckoutSuccessExtender_parent
{
	protected $crConfig;
	protected $crLogger;

	public function proceed()
	{
		parent::proceed();
		$cleverReachInstalled = (bool)gm_get_conf('MODULE_CENTER_CLEVERREACH_INSTALLED') === true;
		if($cleverReachInstalled)
		{
			$this->crConfig = MainFactory::create('CleverReachConfigurationStorage');
			$orderTrackingEnabled = $this->crConfig->get('order_tracking') == true;
			if($orderTrackingEnabled === true)
			{
				$this->crLogger = MainFactory::create('CleverReachLogger');
				$order          = MainFactory::create('order', $this->v_data_array['orders_id']);
				$groups         = array();
				foreach(array('newsletter_group_id', 'allcustomers_group_id', 'allorders_group_id') as $groupIdConfigName)
				{
					$groupId = $this->crConfig->get($groupIdConfigName);
					if($groupId >= 0)
					{
						$groups[$groupIdConfigName] = MainFactory::create('CleverReachGroup', $groupId);
						if(in_array($groupIdConfigName, array('allorders_group_id', 'allcustomers_group_id')))
						{
							$logMessage = sprintf('OrderTracking: Adding receiver %s to group %s (%d)',
								                  $order->customer['email_address'],
								                  $groups[$groupIdConfigName]->name,
								                  $groups[$groupIdConfigName]->id
								                 );
							$this->crLogger->notice($logMessage);
							$this->_addReceiver($order, $groups[$groupIdConfigName]);
						}
					}
				}
				$this->_transmitOrder($groups, $order);
			}
		}
	}

	protected function _addReceiver($order, $group)
	{
		$customerReadService = StaticGXCoreLoader::getService('CustomerRead');
		try
		{
			$customer   = $customerReadService->getCustomerById(MainFactory::create('IdType', $order->customer['id']));
			$dob        = $customer->getDateOfBirth()->format('Y-m-d');
			$date_added = $this->getCustomerDateAdded($customer)->format('Y-m-d');
		}
		catch(InvalidArgumentException $e)
		{
			$dob = '1000-01-01';
			$date_added = '1000-01-01';
		}
		$buyers = array(
			array(
				'email' => (string)$order->customer['email_address'],
				'global_attributes' => array(
					'firstname'  => (string)$order->customer['firstname'],
					'lastname'   => (string)$order->customer['lastname'],
					'street'     => (string)$order->customer['street_address'],
					'zip'        => (string)$order->customer['postcode'],
					'city'       => (string)$order->customer['city'],
					'country'    => (string)$order->customer['country'],
					'company'    => (string)$order->customer['company'],
					'gender'     => (string)$order->customer['gender'],
					'dob'        => $dob,
					'date_added' => $date_added,
				),
			),
		);
		$group->addReceivers($buyers);
	}

	protected function _transmitOrder($groups, $order)
	{
		$mailingId = (isset($_SESSION['crMailingId']) && $_SESSION['crMailingId'] >= 0) ? (int)$_SESSION['crMailingId'] : null;

		foreach($order->products as $product)
		{
			foreach($groups as $crGroup)
			{
				$logMessage = sprintf('OrderTracking %d: Transmitting product %s (%d) for user %s in group %s (%d)',
					                  $order->info['orders_id'],
					                  $product['name'],
					                  $product['id'],
					                  $order->customer['email_address'],
					                  $crGroup->name,
					                  $crGroup->id
					                 );
				$this->crLogger->notice($logMessage);
				$crGroup->addOrder($order->customer['email_address'],
					               $order->info['orders_id'],
					               $product['name'],
					               $product['id'],
					               strtotime($order->info['date_purchased']),
					               $product['price'],
					               $product['qty'],
					               $mailingId,
					               STORE_NAME,
					               $order->info['currency']
					               );
			}
		}
	}

	/**
	 * returns a DateTime representation of customers.customer_date_added
	 *
	 * This is a stand-in for the hitherto missing CustomerInterface::getDateAdded()
	 *
	 * @param $customer CustomerInterface
	 * @return DateTime date added
	 */
	protected function getCustomerDateAdded(CustomerInterface $customer)
	{
		$db = StaticGXCoreLoader::getDatabaseQueryBuilder();
		$query = $db->select('customers_date_added')->where('customers_id', (int)$customer->getId())->get('customers');
		$row = $query->row();
		$customerDateAdded = new DateTime($row->customers_date_added);
		return $customerDateAdded;
	}
}