<?php
/* --------------------------------------------------------------
	CleverReachModuleCenterModuleController.inc.php 2016-04-11
	Gambio GmbH
	http://www.gambio.de
	Copyright (c) 2015 Gambio GmbH
	Released under the GNU General Public License (Version 2)
	[http://www.gnu.org/licenses/gpl-2.0.html]
	--------------------------------------------------------------
*/

class CleverReachModuleCenterModuleController extends AbstractModuleCenterModuleController
{
	// note: StaticGXCoreLoader::getDatabaseQueryBuilder()
	protected $cleverReachText;
	protected $cleverReachConfigurationStorage;
	protected $cleverReachLogger;
	protected $oAuthRedirectURL;
	protected $oAuthScope;

	protected function _init()
	{
		$this->cleverReachText                 = MainFactory::create('CleverReachText');
		$this->cleverReachConfigurationStorage = MainFactory::create('CleverReachConfigurationStorage');
		$this->pageTitle                       = $this->cleverReachText->get_text('configuration_heading');
		$this->cleverReachLogger               = MainFactory::create('CleverReachLogger');
		$this->oAuthScope = 'write';
		$this->oAuthRedirectURL = xtc_href_link('admin.php', 'do=CleverReachModuleCenterModule/StoreToken', 'SSL');
	}

	public function actionDefault()
	{
		$authorizeURL = $this->cleverReachConfigurationStorage->get('oauth/authorization_url') . '?' .
			http_build_query(array(
				'client_id'     => $this->cleverReachConfigurationStorage->get('oauth/client_id'),
				'grant'         => $this->oAuthScope,
				'scope'         => $this->oAuthScope,
				'response_type' => 'code',
				'redirect_uri'  => $this->oAuthRedirectURL,
			));

		$formdata = array(
			'page_token'         => $_SESSION['coo_page_token']->generate_token(),
			'configuration'      => $this->cleverReachConfigurationStorage->get_all(),
			'form_action'        => xtc_href_link('admin.php', 'do=CleverReachModuleCenterModule/SaveConfiguration'),
			'deauthorize_action' => xtc_href_link('admin.php', 'do=CleverReachModuleCenterModule/DeleteToken'),
			'groups'             => false,
			'authorization_link' => $authorizeURL,
			'is_configured'      => false,
		);
		if($this->isConfigured())
		{
			$formdata['is_configured'] = true;
			$formdata['access_until']  = date('c', $this->cleverReachConfigurationStorage->get('oauth/expires_at'));
			$formdata['groups']        = $this->getGroups();
			$formdata['cron_url']      = xtc_href_link('shop.php', 'do=CleverReach/Cron&key=' . $formdata['configuration']['cron_pass']);
		}
		$this->contentView->set_template_dir(DIR_FS_ADMIN . 'html/content/module_center/');
		$html = $this->_render('cleverreach_configuration.html', $formdata);

		return new AdminPageHttpControllerResponse($this->cleverReachText->get_text('configuration_heading'), $html);
	}

	public function actionSaveConfiguration()
	{
		$_SESSION['coo_page_token']->is_valid($this->_getPostData('page_token')) or die('invalid page token');
		$newConfiguration = $this->_getPostData('configuration');
		foreach($newConfiguration as $key => $value)
		{
			$this->cleverReachConfigurationStorage->set($key, $value);
		}
		$GLOBALS['messageStack']->add_session($this->cleverReachText->get_text('configuration_saved'), 'info');

		/*
		try
		{
			$cleverReachGroup = MainFactory::create('CleverReachGroup');
			$addedAttributes = $cleverReachGroup->addRequiredAttributes();
			if(!empty($addedAttributes))
			{
				$GLOBALS['messageStack']->add_session(
					sprintf("%s: %s", $this->cleverReachText->get_text('attributes_added'), implode(', ', $addedAttributes)),
					'info');
			}
		}
		catch (Exception $e)
		{
			$GLOBALS['messageStack']->add_session($e->getMessage(), 'error');
		}
		*/

		if($this->_getPostData('sync_newsletter') !== null)
		{
			$this->syncNewsletter();
		}

		if($this->_getPostData('sync_customers') !== null)
		{
			$this->syncCustomers();
		}

		if($this->_getPostData('sync_orders') !== null)
		{
			$this->syncOrders();
		}

		return new RedirectHttpControllerResponse(xtc_href_link('admin.php', 'do=CleverReachModuleCenterModule'));
	}

	public function actionStoreToken()
	{
		try
		{
			$this->cleverReachLogger->notice('returned from OAuth');
			$code = $this->_getQueryParameter('code');
			if(empty($code))
			{
				throw new Exception('oauth code missing');
			}
			$tokenData = array(
				'grant_type'    => 'authorization_code',
				'code'          => $code,
				'redirect_uri'  => $this->oAuthRedirectURL,
				'client_id'     => $this->cleverReachConfigurationStorage->get('oauth/client_id'),
				'client_secret' => $this->cleverReachConfigurationStorage->get('oauth/client_secret'),
				'scope'         => $this->oAuthScope,
			);
			$restRequest   = MainFactory::create('RestRequest', 'POST', $this->cleverReachConfigurationStorage->get('oauth/token_url'), $tokenData);
			$restService   = MainFactory::create('CleverReachRestService');
			$response      = $restService->performRequest($restRequest);
			$tokenResponse = $response->getResponseObject();
			if($response->getResponseCode() != 200)
			{
				$errorMessage = empty($tokenResponse->error_description) ? 'OAuth Error' : (string)$tokenResponse->error_description;
				throw new Exception($errorMessage);
			}
			$this->cleverReachLogger->debug_notice("OAuth Token Response:\n" . print_r($tokenResponse, true));
			list($token1, $token2) = str_split((string)$tokenResponse->access_token, 255);
			$expiresAt = time() + (int)$tokenResponse->expires_in;
			$this->cleverReachConfigurationStorage->set('oauth/access_token1', $token1);
			$this->cleverReachConfigurationStorage->set('oauth/access_token2', $token2);
			$this->cleverReachConfigurationStorage->set('oauth/expires_at', $expiresAt);
			$this->cleverReachLogger->notice('saved new OAuth Access Token, valid until ' . date('c', $expiresAt));
			$this->addRequiredAttributes();
		}
		catch (Exception $e)
		{
			$GLOBALS['messageStack']->add_session(
				sprintf("%s: %s", $this->cleverReachText->get_text('oauth_error'), $e->getMessage()),
				'error');
		}
		return new RedirectHttpControllerResponse(xtc_href_link('admin.php', 'do=CleverReachModuleCenterModule'));
	}

	public function actionDeleteToken()
	{
		$_SERVER['REQUEST_METHOD'] === 'POST' or die('invalid access method');
		$_SESSION['coo_page_token']->is_valid($this->_getPostData('page_token')) or die('invalid page token');
		$this->cleverReachConfigurationStorage->set('oauth/access_token1', '');
		$this->cleverReachConfigurationStorage->set('oauth/access_token2', '');
		$this->cleverReachConfigurationStorage->set('oauth/expires_at', '');
		$this->cleverReachLogger->notice('Access token deleted');
		return new RedirectHttpControllerResponse(xtc_href_link('admin.php', 'do=CleverReachModuleCenterModule'));
	}

	protected function addRequiredAttributes()
	{
		$requiredAttributes = array(
			'firstname'  => 'text',
			'lastname'   => 'text',
			'street'     => 'text',
			'zip'        => 'text',
			'city'       => 'text',
			'country'    => 'text',
			'company'    => 'text',
			'gender'     => 'gender',
			'dob'        => 'date',
			'date_added' => 'date',
		);

		$crRestService = MainFactory::create('CleverReachRestService');
		$crReadAttributesRequest = MainFactory::create('CleverReachRestRequest', 'GET', '/attributes.json');
		$crGlobalAttributesResponse = $crRestService->performRequest($crReadAttributesRequest);
		$crGlobalAttributes = $crGlobalAttributesResponse->getResponseObject();
		$crGlobalAttributes = is_array($crGlobalAttributes) ? $crGlobalAttributes : array();
		$crGlobalAttributesNames = array();
		foreach($crGlobalAttributes as $globalAttribute)
		{
			$crGlobalAttributesNames[] = $globalAttribute->name;
		}
		$missingAttributes = array_diff(array_keys($requiredAttributes), $crGlobalAttributesNames);
		foreach($missingAttributes as $attributeName)
		{
			$this->cleverReachLogger->notice(sprintf("missing global attribute: %s", $attributeName));
			$newAttribute = array(
				'name' => $attributeName,
				'type' => $requiredAttributes[$attributeName],
			);
			$crNewAttributeRequest = MainFactory::create('CleverReachRestRequest', 'POST', '/attributes.json', json_encode($newAttribute));
			$crNewAttributeResponse = $crRestService->performRequest($crNewAttributeRequest);
			$this->cleverReachLogger->notice(sprintf("created attribute %s:%s", $newAttribute['name'], $newAttribute['type']));
		}
	}

	protected function syncNewsletter()
	{
		try
		{
			$crGroup = MainFactory::create('CleverReachGroup', $this->cleverReachConfigurationStorage->get('newsletter_group_id'));
			$crSync  = MainFactory::create('CleverReachSync');
			$crSync->syncUpstreamQueue($crGroup);
			$crSync->completeNewsletterSync($crGroup);
			$GLOBALS['messageStack']->add_session($this->cleverReachText->get_text('newsletter_sync_completed'), 'info');
		}
		catch (Exception $e)
		{
			$GLOBALS['messageStack']->add_session($e->getMessage(), 'error');
		}
	}

	protected function syncCustomers()
	{
		try
		{
			$crGroup = MainFactory::create('CleverReachGroup', $this->cleverReachConfigurationStorage->get('allcustomers_group_id'));
			$crSync  = MainFactory::create('CleverReachSync');
			$crSync->sendAllCustomers($crGroup);
			$GLOBALS['messageStack']->add_session($this->cleverReachText->get_text('customers_sync_completed'), 'info');
		}
		catch (Exception $e)
		{
			$GLOBALS['messageStack']->add_session($e->getMessage(), 'error');
		}
	}

	protected function syncOrders()
	{
		try
		{
			$crGroup = MainFactory::create('CleverReachGroup', $this->cleverReachConfigurationStorage->get('allorders_group_id'));
			$crSync  = MainFactory::create('CleverReachSync');
			$crSync->sendAllBuyers($crGroup);
			$GLOBALS['messageStack']->add_session($this->cleverReachText->get_text('customers_sync_completed'), 'info');
		}
		catch (Exception $e)
		{
			$GLOBALS['messageStack']->add_session($e->getMessage(), 'error');
		}
	}

	protected function isConfigured()
	{
		$isConfigured = !empty($this->cleverReachConfigurationStorage->get('oauth/access_token1')) &&
		                !empty($this->cleverReachConfigurationStorage->get('oauth/access_token2'));
		return $isConfigured;
	}

	protected function getGroups()
	{
		$groups = false;
		try
		{
			$cleverReachRestService = MainFactory::create('CleverReachRestService');
			$cleverReachRestRequest = MainFactory::create('CleverReachRestRequest', 'GET', '/groups.json');
			$response = $cleverReachRestService->performRequest($cleverReachRestRequest);
			$groups = $response->getResponseObject();
		}
		catch(Exception $e)
		{
			$GLOBALS['messageStack']->add_session($e->getMessage(), 'error');
		}
		return $groups;
	}

}