<?php
/* --------------------------------------------------------------
	CleverReachConfigurationStorage.inc.php 2016-04-12
	Gambio GmbH
	http://www.gambio.de
	Copyright (c) 2015 Gambio GmbH
	Released under the GNU General Public License (Version 2)
	[http://www.gnu.org/licenses/gpl-2.0.html]
	--------------------------------------------------------------
*/

class CleverReachConfigurationStorage extends ConfigurationStorage
{
	/**
	 * namespace inside the configuration storage
	 */
	const CONFIG_STORAGE_NAMESPACE = 'modules/tracking/cleverreach';

	/**
	 * array holding default values to be used in absence of configured values
	 */
	protected $default_configuration;

	/**
	 * constructor; initializes default configuration
	 */
	public function __construct()
	{
		parent::__construct(self::CONFIG_STORAGE_NAMESPACE);
		$this->setDefaultConfiguration();
	}

	/**
	 * fills $default_configuration with initial values
	 */
	protected function setDefaultConfiguration()
	{
		$this->default_configuration = array(
			'cron_pass'               => uniqid(),
			'debug_logging'           => '1',
			'order_tracking'          => '0',
			'newsletter_group_id'     => '-1',
			'allcustomers_group_id'   => '-1',
			'allorders_group_id'      => '-1',
			'prio'                    => 'cleverreach',
			'service_base_url'        => 'https://rest.cleverreach.com/v1',
			'oauth/client_id'         => 'pMR9Ca0Ckf',
			'oauth/client_secret'     => 'BIHhWiWMryje4gmtGg22SrPJAsTB5eQG',
			'oauth/access_token1'	  => '',
			'oauth/access_token2'	  => '',
			'oauth/expires_at'        => '0',
			'oauth/authorization_url' => 'https://rest.cleverreach.com/oauth/authorize.php',
			'oauth/token_url'         => 'https://rest.cleverreach.com/oauth/token.php',
			'mailing_id_parameter'    => 'crmailing',
		);
	}

	/**
	 * returns a single configuration value by its key
	 * @param string $key a configuration key (relative to the namespace prefix)
	 * @return string configuration value
	 */
	public function get($key)
	{
		$value = parent::get($key);
		if($value === false && array_key_exists($key, $this->default_configuration))
		{
			$value = $this->default_configuration[$key];
		}
		return $value;
	}

	public function get_all($p_prefix = '')
	{
		$values = parent::get_all($p_prefix);
		if(empty($values))
		{
			foreach($this->default_configuration as $key => $default_value)
			{
				$key_prefix = substr($key, 0, strlen($p_prefix));
				if($key_prefix == $p_prefix)
				{
					$values[$key] = $default_value;
				}
			}
		}

		return $values;
	}

	/**
	 * stores a configuration value by name/key
	 * @param string $name name/key of configuration entry
	 * @param string $value value to be stored
	 * @throws Exception if data validation fails
	 */
	public function set($name, $value)
	{
		if($value === null)
		{
			return;
		}

		switch($name)
		{
			case 'prio':
				$value = in_array($value, array('webshop', 'cleverreach')) ? $value : 'cleverreach';
				break;
			case 'newsletter_group_id':
				$value = (int)$value;
				break;
			case 'cron_pass':
				$value = urlencode(strip_tags($value));
				break;
			case 'debug_logging':
			case 'order_tracking':
				$value = (bool)$value === true ? '1' : '0';
				break;
			default:
				$value = strip_tags($value);
		}
		parent::set($name, $value);
	}

}