<?php
/* --------------------------------------------------------------
	CleverReachNewsletterExtender.inc.php 2016-03-16
	Gambio GmbH
	http://www.gambio.de
	Copyright (c) 2015 Gambio GmbH
	Released under the GNU General Public License (Version 2)
	[http://www.gnu.org/licenses/gpl-2.0.html]
	--------------------------------------------------------------
*/

class CleverReachNewsletterExtender extends CleverReachNewsletterExtender_parent
{
	protected $db;
	protected $logger;
	protected $enabled;

	public function proceed()
	{
		$this->enabled = (bool)gm_get_conf('MODULE_CENTER_CLEVERREACH_INSTALLED') === true;
		if ($this->enabled === true)
		{
			$this->db = StaticGXCoreLoader::getDatabaseQueryBuilder();
			$this->logger = MainFactory::create('CleverReachLogger');
		}
	}

	public function unsubscribeNewsletter(CustomerEmail $emailAddress)
	{
		if ($this->enabled === true)
		{
			$this->logger->notice('unsubscribing ' . $emailAddress);
			try
			{
				$syncId = $this->_addToQueue($emailAddress, 'DEL');
				$this->_syncNow($syncId);
			}
			catch (NewsletterSubscriptionExtenderComponentCustomerNotFoundException $e)
			{
				$this->logger->notice('cannot unsubscribe: customer not found, email address invalid: ' . $emailAddress);
			}
			catch (Exception $e)
			{
				$this->logger->notice('cannot unsubscribe' . $emailAddress . ', ERROR: ' . $e->getMessage());
			}
		}
	}

	public function subscribeNewsletter(CustomerEmail $emailAddress)
	{
		if ($this->enabled === true)
		{
			$this->logger->notice('subscribing ' . $emailAddress);
			try
			{
				$syncId = $this->_addToQueue($emailAddress, 'ADD');
				$this->_syncNow($syncId);
			}
			catch (NewsletterSubscriptionExtenderComponentCustomerNotFoundException $e)
			{
				$this->logger->notice('cannot subscribe: customer not found, email address invalid: ' . $emailAddress);
			}
			catch (Exception $e)
			{
				$this->logger->notice('cannot subscribe' . $emailAddress . ', ERROR: ' . $e->getMessage());
			}
		}
	}

	protected function _addToQueue($emailAddress, $operation)
	{
		$customer = $this->getCustomerByEmail($emailAddress);
		$crSyncData = array(
			'customers_id' => $customer->getId(),
			'email'        => (string)$emailAddress,
			'action'       => $operation,
			'status'       => 0,
			'created_at'   => date('Y-m-d H:i:s'),
		);
		$this->db->insert('cleverreach_api_sync', $crSyncData);
		$syncId = $this->db->insert_id();
		return $syncId;
	}

	protected function _syncNow($syncId)
	{
		$crGroup = MainFactory::create('CleverReachGroup');
		$crSync = MainFactory::create('CleverReachSync');
		$crSync->syncUpstreamQueue($syncId, $crGroup);
	}
}

